#[non_exhaustive]pub enum Command {
AutoOrient(u8),
Rotate(Rotation),
Flip(FlipAxis),
Crop(SourceCrop),
Region(Region),
Constrain(Constraint),
Pad(Padding),
}Expand description
A single image processing command.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
AutoOrient(u8)
Apply EXIF orientation correction (value 1-8).
Rotate(Rotation)
Manual rotation. Composes with all orientation commands into a single source transform.
Flip(FlipAxis)
Manual flip. Composes with all orientation commands into a single source transform.
Crop(SourceCrop)
Crop in post-orientation coordinates.
Region(Region)
Viewport region in post-orientation coordinates. Unifies crop and pad.
Constrain(Constraint)
Constrain dimensions in post-orientation coordinates.
Pad(Padding)
Add padding around the image.